import { Injectable } from '@nestjs/common';
import { ConfigService } from '@nestjs/config';
import { IDefaultParams } from '../../domain/types/default-params.type';

@Injectable()
export class DefaultParamsProvider {
  constructor(private readonly configService: ConfigService) {}

  getParams = (): IDefaultParams => {
    return {
      DEFAULT_UUID: this.configService.get('DEFAULT_UUID'),
      MARGIN_DURATION: parseInt(this.configService.get('MARGIN_DURATION')),
      VALIDITY_DURATION: parseInt(this.configService.get('VALIDITY_DURATION')),
      DEFAULT_TIMEZONE: this.configService.get('DEFAULT_TIMEZONE'),
      DEFAULT_SEATS: parseInt(this.configService.get('DEFAULT_SEATS')),
      DEFAULT_ALGORITHM_SETTINGS: {
        algorithm: this.configService.get('ALGORITHM'),
        strict: !!parseInt(this.configService.get('STRICT_ALGORITHM')),
        remoteness: parseInt(this.configService.get('REMOTENESS')),
        useProportion: !!parseInt(this.configService.get('USE_PROPORTION')),
        proportion: parseInt(this.configService.get('PROPORTION')),
        useAzimuth: !!parseInt(this.configService.get('USE_AZIMUTH')),
        azimuthMargin: parseInt(this.configService.get('AZIMUTH_MARGIN')),
        maxDetourDistanceRatio: parseFloat(
          this.configService.get('MAX_DETOUR_DISTANCE_RATIO'),
        ),
        maxDetourDurationRatio: parseFloat(
          this.configService.get('MAX_DETOUR_DURATION_RATIO'),
        ),
        georouterType: this.configService.get('GEOROUTER_TYPE'),
        georouterUrl: this.configService.get('GEOROUTER_URL'),
      },
    };
  };
}
