import { IRequestAlgorithmSettings } from '../../interfaces/algorithm-settings-request.interface';
import { DefaultAlgorithmSettings } from '../../types/default-algorithm-settings.type';
import { AlgorithmType } from '../../types/algorithm.enum';
import { ICreateGeorouter } from '../../interfaces/georouter-creator.interface';
import { IGeorouter } from '../../interfaces/georouter.interface';
import { Frequency } from '../../../../ad/domain/types/frequency.enum';

export class AlgorithmSettings {
  private algorithmSettingsRequest: IRequestAlgorithmSettings;
  private strict: boolean;
  algorithmType: AlgorithmType;
  restrict: Frequency;
  remoteness: number;
  useProportion: boolean;
  proportion: number;
  useAzimuth: boolean;
  azimuthMargin: number;
  maxDetourDurationRatio: number;
  maxDetourDistanceRatio: number;
  georouter: IGeorouter;

  constructor(
    algorithmSettingsRequest: IRequestAlgorithmSettings,
    defaultAlgorithmSettings: DefaultAlgorithmSettings,
    frequency: Frequency,
    georouterCreator: ICreateGeorouter,
  ) {
    this.algorithmSettingsRequest = algorithmSettingsRequest;
    this.algorithmType =
      algorithmSettingsRequest.algorithm ?? defaultAlgorithmSettings.algorithm;
    this.strict =
      algorithmSettingsRequest.strict ?? defaultAlgorithmSettings.strict;
    this.remoteness = algorithmSettingsRequest.remoteness
      ? Math.abs(algorithmSettingsRequest.remoteness)
      : defaultAlgorithmSettings.remoteness;
    this.useProportion =
      algorithmSettingsRequest.useProportion ??
      defaultAlgorithmSettings.useProportion;
    this.proportion = algorithmSettingsRequest.proportion
      ? Math.abs(algorithmSettingsRequest.proportion)
      : defaultAlgorithmSettings.proportion;
    this.useAzimuth =
      algorithmSettingsRequest.useAzimuth ??
      defaultAlgorithmSettings.useAzimuth;
    this.azimuthMargin = algorithmSettingsRequest.azimuthMargin
      ? Math.abs(algorithmSettingsRequest.azimuthMargin)
      : defaultAlgorithmSettings.azimuthMargin;
    this.maxDetourDistanceRatio =
      algorithmSettingsRequest.maxDetourDistanceRatio ??
      defaultAlgorithmSettings.maxDetourDistanceRatio;
    this.maxDetourDurationRatio =
      algorithmSettingsRequest.maxDetourDurationRatio ??
      defaultAlgorithmSettings.maxDetourDurationRatio;
    this.georouter = georouterCreator.create(
      defaultAlgorithmSettings.georouterType,
      defaultAlgorithmSettings.georouterUrl,
    );
    if (this.strict) {
      this.restrict = frequency;
    }
  }
}
