import { Injectable } from '@nestjs/common';
import { MatchQuery } from '../../../queries/match.query';
import { Match } from '../ecosystem/match';
import { Candidate } from './candidate';
import { AlgorithmFactory } from './factory/algorithm-factory.abstract';
import { AlgorithmFactoryCreator } from './factory/algorithm-factory-creator';

@Injectable()
export class Matcher {
  constructor(
    private readonly algorithmFactoryCreator: AlgorithmFactoryCreator,
  ) {}

  match = async (matchQuery: MatchQuery): Promise<Match[]> => {
    const algorithmFactory: AlgorithmFactory =
      this.algorithmFactoryCreator.create(matchQuery);
    let candidates: Candidate[] = await algorithmFactory
      .createSelector()
      .select();
    for (const processor of algorithmFactory.createProcessors()) {
      candidates = processor.execute(candidates);
    }
    const match = new Match();
    match.uuid = 'e23f9725-2c19-49a0-9ef6-17d8b9a5ec85';
    return [match];
  };
}
